﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace InfragisticsChartPerf
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Window1_Loaded);
        }

        System.Windows.Threading.DispatcherTimer timer = new System.Windows.Threading.DispatcherTimer();
        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            textBoxX.Text = "25";
            textBoxY.Text = "25";

            IntensityChart.Scene.GridArea.RenderingOptions.RenderingMode = Infragistics.Windows.Chart.RenderingMode.Performance;
            UpdateIntensityChart();
            IntensityChart.Legend.Visible = false;

            // Rotate the chart so that it is the correct orientation
            System.Windows.Media.Media3D.QuaternionRotation3D oQR3D = new System.Windows.Media.Media3D.QuaternionRotation3D();
            double dZRadians = 90 * (Math.PI / 180);
            double dXRadians = 45 * (Math.PI / 180);
            double dYRadians = 20 * (Math.PI / 180);
            oQR3D.Quaternion = new System.Windows.Media.Media3D.Quaternion(Math.Sin(dXRadians / 2), Math.Sin(dYRadians / 2), Math.Sin(dZRadians / 2), Math.Cos(dZRadians / 2));
            System.Windows.Media.Media3D.RotateTransform3D oRT3D = new System.Windows.Media.Media3D.RotateTransform3D(oQR3D);
            IntensityChart.Transform3D = oRT3D;
        }

        private void ComputeRotation()
        {
            double dXRadians = 2 * (Math.PI / 180);
            double dYRadians = 2 * (Math.PI / 180);

            // Limit the rotation to keep the chart in view
            System.Windows.Media.Media3D.RotateTransform3D oRT3D = (System.Windows.Media.Media3D.RotateTransform3D)IntensityChart.Transform3D;
            System.Windows.Media.Media3D.Quaternion oQ = ((System.Windows.Media.Media3D.QuaternionRotation3D)oRT3D.Rotation).Quaternion;
            if (Math.Asin(oQ.X) + dXRadians < 0) dXRadians = 0;
            if (Math.Asin(oQ.Y) + dYRadians < 0) dYRadians = 0;
            if (Math.Asin(oQ.X) + dXRadians > 45 * (Math.PI / 180)) dXRadians = 0;
            if (Math.Asin(oQ.Y) + dYRadians > .34) dYRadians = 0;

            // Build the new rotation and apply
            System.Windows.Media.Media3D.Quaternion oNewQ = new System.Windows.Media.Media3D.Quaternion(Math.Sin(Math.Asin(oQ.X) + dXRadians), Math.Sin(Math.Asin(oQ.Y) + dYRadians), oQ.Z, oQ.W);
            System.Windows.Media.Media3D.QuaternionRotation3D oQR3D = new System.Windows.Media.Media3D.QuaternionRotation3D();
            oQR3D.Quaternion = oNewQ;
            IntensityChart.Transform3D = new System.Windows.Media.Media3D.RotateTransform3D(oQR3D);

        }

        public void UpdateIntensityChart()
        {
            DataTable oDT = BuildChartData();

            // Bind the data to the existing series
            // Build the series
            IntensityChart.RefreshEnabled = false;
            IntensityChart.Series.Clear();
            foreach (DataColumn oDC in oDT.Columns)
            {
                Infragistics.Windows.Chart.Series oS = new Infragistics.Windows.Chart.Series();
                oS.DataMapping = oDC.ColumnName;
                oS.Label = oDC.ColumnName;
                oS.DataSource = oDT;
                oS.ChartType = Infragistics.Windows.Chart.ChartType.Bar;
                IntensityChart.Series.Add(oS);
            }

            IntensityChart.RefreshEnabled = true;
        }

        private DataTable BuildChartData()
        {
            // Create a datatable and add columns to it
            DataTable oDT = new DataTable();
            for (double iCol = 0; iCol < Convert.ToDouble(textBoxX.Text); iCol++)
            {
                string strColName = string.Format("X{0}", iCol);
                oDT.Columns.Add(new DataColumn(strColName, typeof(float)));
            }

            // Add rows (Y coord), with the intensity values

            for (int iRow = 0; iRow < Convert.ToInt32(textBoxY.Text); iRow++)
            {
                DataRow oDRow = oDT.NewRow();

                for (int iJIdx = 0; iJIdx < Convert.ToInt32(textBoxX.Text); iJIdx++)
                {
                    oDRow[iJIdx] = 120;
                }

                oDT.Rows.Add(oDRow);
            }

            return oDT;
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            UpdateIntensityChart();
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            ComputeRotation();
        }
    }
}
